#include <stdlib.h>
#include "perlDocumentHandler.hpp"
#include "util/XMLString.hpp"
#include "sax/AttributeList.hpp"

perlDocumentHandler::perlDocumentHandler() {
    startElementFunction = NULL;
    charactersFunction = NULL;
    ignorableWhitespaceFunction = NULL;
}

perlDocumentHandler::~perlDocumentHandler() {
    if (startElementFunction) {
	SvREFCNT_dec(startElementFunction); 
    }
    if (ignorableWhitespaceFunction) {
	SvREFCNT_dec(ignorableWhitespaceFunction); 
    }
    if (charactersFunction) {
	SvREFCNT_dec(charactersFunction); 
    }
}

void
perlDocumentHandler::setIgnorableWhitespaceFunction(SV *function) {
    if (SvTYPE( SvRV(function)) == SVt_PVCV) {
        SvREFCNT_inc(function);
        ignorableWhitespaceFunction = function;
    } else {
        fprintf(stderr, "setIgnorableWhitespaceFunction:  invalid argument for setCharacterFunction, must be code reference\n");
    }
}

void
perlDocumentHandler::setCharactersFunction(SV *function) {
    if (SvTYPE( SvRV(function)) == SVt_PVCV) {
        SvREFCNT_inc(function);
        charactersFunction = function;
    } else {
        fprintf(stderr, "setCharactersFunction:  invalid argument for setCharacterFunction, must be code reference\n");
    }
}

void
perlDocumentHandler::setStartElementFunction(SV *function) {
    if (SvTYPE( SvRV(function)) == SVt_PVCV) {
        SvREFCNT_inc(function);
        startElementFunction = function;
    } else {
        fprintf(stderr, "StartElement:  invalid argument for set_StartElementFunction, must be code reference\n");
    }
}

void 
perlDocumentHandler::startElement(const XMLCh* const name, 
				  AttributeList& attributes) {
    if (!startElementFunction) return;
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
        // first argument is the element name
    char *cptr = XMLString::transcode(name);
    SV *string = sv_newmortal();
    sv_setpv(string, (char *)cptr);
    XPUSHs(string);

        // next is the attribute list
    char *class_name = "XML::Xerces::AttributeList";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&attributes), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));
    PUTBACK;

    perl_call_sv(startElementFunction, G_VOID);

    FREETMPS;
    LEAVE;
}

void
perlDocumentHandler::characters(const XMLCh* const chars, 
				const unsigned int length) 
{
    if (!charactersFunction) return;
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
        // first argument is the element name
    char *cptr = XMLString::transcode(chars);
    SV *string = sv_newmortal();
    sv_setpv(string, (char *)cptr);
    XPUSHs(string);

        // next is the length
    XPUSHs(sv_2mortal(newSViv(length)));
    PUTBACK;

    perl_call_sv(charactersFunction, G_VOID);

    FREETMPS;
    LEAVE;

}
void
perlDocumentHandler::ignorableWhitespace(const XMLCh* const chars, 
					 const unsigned int length) 
{
    if (!ignorableWhitespaceFunction) return;
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
        // first argument is the element name
    char *cptr = XMLString::transcode(chars);
    SV *string = sv_newmortal();
    sv_setpv(string, (char *)cptr);
    XPUSHs(string);

        // next is the length
    XPUSHs(sv_2mortal(newSViv(length)));
    PUTBACK;

    perl_call_sv(ignorableWhitespaceFunction, G_VOID);

    FREETMPS;
    LEAVE;

}
void perlDocumentHandler::resetDocument(void){}


