class perlErrorHandler : public ErrorHandler {

//private:
//
//	SV *warningFunction;
//	SV *errorFunction;
//	SV *fatalErrorFunction;
//

public:

	perlErrorHandler() {
		warningFunction = errorFunction = fatalErrorFunction = NULL;
	}

	~perlErrorHandler() {
		if (warningFunction)	{ SvREFCNT_dec(warningFunction); }
		if (errorFunction)	{ SvREFCNT_dec(errorFunction); }
		if (fatalErrorFunction)	{ SvREFCNT_dec(fatalErrorFunction); }
	}

	void setWarningFunction(SV *function) {
		if (  SvTYPE( SvRV(function) ) == SVt_PVCV  ) {
			SvREFCNT_inc(function);
			warningFunction = function;
		} else {
			fprintf(stderr, "Warning:  invalid argument for set_warning_function, must be code reference\n");
		}
	}

	void setErrorFunction(SV *function) {
		if (  SvTYPE( SvRV(function) ) == SVt_PVCV  ) {
			SvREFCNT_inc(function);
			errorFunction = function;
		} else {
			fprintf(stderr, "Warning:  invalid argument for set_error_function, must be code reference\n");
		}
	}

	void setFatalErrorFunction(SV *function) {
		if (  SvTYPE( SvRV(function) ) == SVt_PVCV  ) {
			SvREFCNT_inc(function);
			fatalErrorFunction = function;
		} else {
			fprintf(stderr, "Warning:  invalid argument for set_fatal_error_function, must be code reference\n");
		}
	}

	virtual void warning(const SAXParseException& exception) {

		if (!warningFunction) return;

		dSP;

		ENTER;
		SAVETMPS;

		PUSHMARK(SP);
		HV *hash = newHV();
		HV *stash = gv_stashpv("SAXParseException", FALSE);
		hv_magic(  hash, (GV *)sv_setref_pv( sv_newmortal(), "SAXParseException", (void *)&exception ), 'P'  );
		XPUSHs(   sv_bless(  newRV_noinc( (SV *)hash ), stash  )   );
		PUTBACK;

		perl_call_sv(warningFunction, G_VOID);

		FREETMPS;
		LEAVE;
	}

	virtual void error(const SAXParseException& exception) {

		if (!errorFunction) return;

		dSP;

		ENTER;
		SAVETMPS;

		PUSHMARK(SP);
		HV *hash = newHV();
		HV *stash = gv_stashpv("SAXParseException", FALSE);
		hv_magic(  hash, (GV *)sv_setref_pv( sv_newmortal(), "SAXParseException", (void *)&exception ), 'P'  );
		XPUSHs(   sv_bless(  newRV_noinc( (SV *)hash ), stash  )   );
		PUTBACK;

		perl_call_sv(errorFunction, G_VOID);

		FREETMPS;
		LEAVE;
	}

	virtual void fatalError(const SAXParseException& exception) {

		if (!fatalErrorFunction) return;

		dSP;

		ENTER;
		SAVETMPS;

		PUSHMARK(SP);
		HV *hash = newHV();
		HV *stash = gv_stashpv("SAXParseException", FALSE);
		hv_magic(  hash, (GV *)sv_setref_pv( sv_newmortal(), "SAXParseException", (void *)&exception ), 'P'  );
		XPUSHs(   sv_bless(  newRV_noinc( (SV *)hash ), stash  )   );
		PUTBACK;

		perl_call_sv(fatalErrorFunction, G_VOID);

		FREETMPS;
		LEAVE;
	}

};

